local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local QuestingModule = idTweaks:NewModule('Questing', 'AceConsole-3.0', 'AceEvent-3.0')

function QuestingModule:setDefaults()
  --QuestingModule:Print('QuestingModule:setDefaults')

  self.defaults = {
    profile = {
      pauseKey = 'shift',
      acceptQuests = true,
      completeQuests = true,
    }
  }
end

function QuestingModule:setDatabase()
  --QuestingModule:Print('QuestingModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Questing', self.defaults)
end

function QuestingModule:setOptions()
  -- QuestingModule:Print('QuestingModule:setOptions')

  self.options = {
    name = 'Questing',
    desc = 'Options for questing.',
    handler = self,
    type = 'group',
    args = {
      pauseKey = {
        order = 1,
        name = 'Pause Key',
        desc = 'While you hold this key, nothing will happen, so you can read your quest in peace.',
        handler = self,
        type = 'select',
        values = {
          shift = 'Shift',
          control = 'Control',
          alt = 'Alt',
        },
        get = function()
          return self.db.profile.pauseKey
        end,
        set = function(_, value)
          self.db.profile.pauseKey = value
        end
      },
      acceptQuests = {
        order = 2,
        name = 'Accept Quests',
        desc = 'Automatically accept all quests. Might need a few clicks to get them all.',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.acceptQuests
        end,
        set = function(_, value)
          self.db.profile.acceptQuests = value
        end
      },
      completeQuests = {
        order = 3,
        name = 'Complete quests',
        desc = 'Automatically complete all quests that dont require you to pick a reward.',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.completeQuests
        end,
        set = function(_, value)
          self.db.profile.completeQuests = value
        end
      }
    },
  }
end

function QuestingModule:registerOptions()
  --QuestingModule:Print('QuestingModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('Questing', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('Questing', 'Questing', 'idTweaks')
end

function QuestingModule:OnInitialize()
  --QuestingModule:Print('QuestingModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function QuestingModule:OnEnable()
  --QuestingModule:Print('QuestingModule:OnEnable')

  self:registerOptions()

  -- Register for events
  self:RegisterEvent('GOSSIP_SHOW')
  self:RegisterEvent('QUEST_AUTOCOMPLETE')
  self:RegisterEvent('QUEST_COMPLETE')
  self:RegisterEvent('QUEST_DETAIL')
  self:RegisterEvent('QUEST_GREETING')
  self:RegisterEvent('QUEST_LOG_UPDATE')
  self:RegisterEvent('QUEST_PROGRESS')
end

function QuestingModule:OnDisable()
  --QuestingModule:Print('QuestingModule:OnDisable')
end

function QuestingModule:shouldAutomate()
  local key = self.db.profile.pauseKey

  if key == 'shift' then
    if IsShiftKeyDown() then
      return false
    end
  end

  if key == 'control' then
    if IsControlKeyDown() then
      return false
    end
  end

  if key == 'alt' then
    if IsAltKeyDown() then
      return false
    end
  end

  return true
end

function QuestingModule:shouldAccept()
  -- QuestingModule:Print('QuestingModule:shouldAccept', self.db.profile.shouldAccept)

  return not not self.db.profile.acceptQuests
end

function QuestingModule:shouldComplete()
  -- QuestingModule:Print('QuestingModule:shouldComplete', self.db.profile.shouldComplete)

  return not not self.db.profile.completeQuests
end

function QuestingModule:stripText(text)
  if not text then return end
  text = text:gsub('|c%x%x%x%x%x%x%x%x(.-)|r','%1')
  text = text:gsub('%[.*%]%s*','')
  text = text:gsub('(.+) %(.+%)', '%1')
  text = text:trim()
  return text
end

function QuestingModule:QUEST_AUTOCOMPLETE(event, questId)
  -- QuestingModule:Print('QuestingModule:QUEST_AUTOCOMPLETE')

  if not self:shouldAutomate() then return end
  if not self:shouldComplete() then return end
  for i = 0, 6 do
    local id = GetQuestIndexForWatch(i)
    if id then
      ShowQuestComplete(id)
    end
  end
end

function QuestingModule:QUEST_COMPLETE(event)
  -- QuestingModule:Print('QuestingModule:QUEST_COMPLETE')

  if not self:shouldAutomate() then return end
  if not self:shouldComplete() then return end
  if GetNumQuestChoices() <= 1 then
    QuestFrameCompleteQuestButton:Click()
  end
end

function QuestingModule:QUEST_PROGRESS()
  -- QuestingModule:Print('QuestingModule:QUEST_PROGRESS')

  if not self:shouldAutomate() then return end
  if not self:shouldComplete() then return end
  if IsQuestCompletable() then
    CompleteQuest()
  end
end

function QuestingModule:QUEST_LOG_UPDATE()
  -- QuestingModule:Print('QuestingModule:QUEST_LOG_UPDATE')

  local startEntry = GetQuestLogSelection()
  local numEntries = GetNumQuestLogEntries()
  local title
  local isComplete
  local noObjectives
  local _

  self.completedQuests = {}
  self.incompleteQuests = {}

  if numEntries > 0 then
    for i = 1, numEntries do
      SelectQuestLogEntry(i)
      title, _, _, _, _, isComplete = GetQuestLogTitle(i)
      if title then
        if isComplete then
          self.completedQuests[title] = true
        else
          self.incompleteQuests[title] = true
        end
      end
    end
  end

  SelectQuestLogEntry(startEntry)
end

function QuestingModule:GOSSIP_SHOW()
  -- QuestingModule:Print('QuestingModule:GOSSIP_SHOW')

  if not self:shouldAutomate() then return end

  local button
  local text

  if
    GossipTitleButton1:IsVisible() and
    GossipTitleButton1.type == 'Gossip' and
    not GossipTitleButton2:IsVisible()
  then
    GossipTitleButton1:Click()
    return
  end

  for i = 32, 1, -1 do
    button = _G['GossipTitleButton' .. i]

    if button:IsVisible() then
      text = self:stripText(button:GetText())

      if
        button.type == 'Available' and
        self:shouldAccept()
      then
        button:Click()
      elseif
        button.type == 'Active' and
        self:shouldComplete() and
        self.completedQuests[text]
      then
        button:Click()
      end
    end
  end
end

function QuestingModule:QUEST_GREETING(...)
  -- QuestingModule:Print('QuestingModule:QUEST_GREETING')

  if not self:shouldAutomate() then return end

  local button
  local text

  for i = 32, 1, -1 do
    button = _G['QuestTitleButton' .. i]

    if button and button:IsVisible() then
      text = self:stripText(button:GetText())
      if
        self.completedQuests[text] and
        self:shouldComplete()
      then
        button:Click()
      elseif
        not self.incompleteQuests[text] and
        self:shouldAccept()
      then
        button:Click()
      end
    end
  end
end

function QuestingModule:QUEST_DETAIL()
  -- QuestingModule:Print('QuestingModule:QUEST_DETAIL')

  if not self:shouldAutomate() then return end
  if not self:shouldAccept() then return end

  AcceptQuest()
end

idTweaks.Questing = QuestingModule
